@@ -1,7 +1,7 @@
 #ifndef __pform_H
 #define __pform_H
 /*
- * Copyright (c) 1998-2008 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2011 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -35,7 +35,7 @@
 # include  <iostream>
 # include  <string>
 # include  <list>
-# include  <stdio.h>
+# include  <cstdio>
 
 /*
  * These classes implement the parsed form (P-form for short) of the
@@ -159,7 +159,8 @@ extern void pform_module_define_port(const struct vlltype&li,
 extern Module::port_t* pform_module_port_reference(perm_string name,
 						   const char*file,
 						   unsigned lineno);
-extern void pform_endmodule(const char*);
+extern void pform_endmodule(const char*, bool inside_celldefine,
+                            Module::UCDriveType uc_drive_def);
 
 extern void pform_make_udp(perm_string name, list<perm_string>*parms,
 			   svector<PWire*>*decl, list<string>*table,
@@ -180,8 +181,10 @@ extern void pform_make_udp(perm_string name,
  */
 extern void pform_pop_scope();
 
-extern PTask*pform_push_task_scope(char*name, bool is_auto);
-extern PFunction*pform_push_function_scope(char*name, bool is_auto);
+extern PTask*pform_push_task_scope(const struct vlltype&loc, char*name,
+                                   bool is_auto);
+extern PFunction*pform_push_function_scope(const struct vlltype&loc, char*name,
+                                           bool is_auto);
 extern PBlock*pform_push_block_scope(char*name, PBlock::BL_TYPE tt);
 
 extern void pform_put_behavior_in_scope(AProcess*proc);
@@ -191,7 +194,7 @@ extern verinum* pform_verinum_with_size(verinum*s, verinum*val,
 
 /*
  * This function takes the list of names as new genvars to declare in
- * the current module scope.
+ * the current module or generate scope.
  */
 extern void pform_genvars(const struct vlltype&li, list<perm_string>*names);
 
@@ -353,7 +356,8 @@ extern svector<PWire*>*pform_make_task_ports(NetNet::PortType pt,
 					     svector<PExpr*>*range,
 					     list<perm_string>*names,
 					     const char* file,
-					     unsigned lineno);
+					     unsigned lineno,
+					     bool isint = false);
 
 
 /*
