@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -20,9 +20,6 @@
 # include  "vvp_priv.h"
 # include  <string.h>
 # include  <assert.h>
-#ifdef HAVE_MALLOC_H
-# include  <malloc.h>
-#endif
 # include  <stdlib.h>
 
 #ifdef __MINGW32__ /* MinGW has inconsistent %p output. */
@@ -42,22 +39,6 @@ static unsigned transient_id = 0;
  * executable code for the processes.
  */
 
-unsigned bitchar_to_idx(char bit)
-{
-      switch (bit) {
-	  case '0':
-	    return 0;
-	  case '1':
-	    return 1;
-	  case 'x':
-	    return 2;
-	  case 'z':
-	    return 3;
-	  default:
-	    assert(0);
-	    return 0;
-      }
-}
 
 /*
  * These functions handle the blocking assignment. Use the %set
@@ -87,7 +68,8 @@ static void set_to_lvariable(ivl_lval_t lval,
 
       if (part_off_ex == 0) {
 	    part_off = 0;
-      } else if (number_is_immediate(part_off_ex, IMM_WID, 0)) {
+      } else if (number_is_immediate(part_off_ex, IMM_WID, 0) &&
+                 !number_is_unknown(part_off_ex)) {
 	    part_off = get_number_immediate(part_off_ex);
 	    part_off_ex = 0;
       }
@@ -96,6 +78,7 @@ static void set_to_lvariable(ivl_lval_t lval,
 	   it to select the word, and pay no further heed to the
 	   expression itself. */
       if (word_ix && number_is_immediate(word_ix, IMM_WID, 0)) {
+	    assert(! number_is_unknown(word_ix));
 	    use_word = get_number_immediate(word_ix);
 	    word_ix = 0;
       }
@@ -233,12 +216,14 @@ static void assign_to_array_word(ivl_signal_t lsig, ivl_expr_t word_ix,
       if (part_off_ex == 0) {
 	    part_off = 0;
       } else if (number_is_immediate(part_off_ex, IMM_WID, 0)) {
+	    assert(! number_is_unknown(part_off_ex));
 	    part_off = get_number_immediate(part_off_ex);
 	    part_off_ex = 0;
       }
 
 	/* This code is common to all the different types of array delays. */
       if (number_is_immediate(word_ix, IMM_WID, 0)) {
+	    assert(! number_is_unknown(word_ix));
 	    fprintf(vvp_out, "    %%ix/load 3, %lu, 0; address\n",
 	                     get_number_immediate(word_ix));
       } else {
@@ -317,6 +302,7 @@ static void assign_to_lvector(ivl_lval_t lval, unsigned bit,
       if (part_off_ex == 0) {
 	    part_off = 0;
       } else if (number_is_immediate(part_off_ex, IMM_WID, 0)) {
+	    assert(! number_is_unknown(part_off_ex));
 	    part_off = get_number_immediate(part_off_ex);
 	    part_off_ex = 0;
       }
@@ -575,7 +561,7 @@ static int show_stmt_assign_sig_real(ivl_statement_t net)
 
 	// For now, only support 1-dimensional arrays.
       assert(ivl_signal_dimensions(var) == 1);
- 
+
 	// Calculate the word index into an index register
       ivl_expr_t word_ex = ivl_lval_idx(lval);
       int word_ix = allocate_word();
@@ -597,19 +583,11 @@ static int show_stmt_assign(ivl_statement_t net)
       lval = ivl_stmt_lval(net, 0);
 
       sig = ivl_lval_sig(lval);
-      if (sig) switch (ivl_signal_data_type(sig)) {
-
-	  case IVL_VT_REAL:
+      if (sig && (ivl_signal_data_type(sig) == IVL_VT_REAL)) {
 	    return show_stmt_assign_sig_real(net);
-
-	  default:
-	    return show_stmt_assign_vector(net);
-
-      } else {
-	    return show_stmt_assign_vector(net);
       }
 
-      return 0;
+      return show_stmt_assign_vector(net);
 }
 
 /*
@@ -647,7 +625,15 @@ static int show_stmt_assign_nb_real(ivl_statement_t net)
 	    word_ix = ivl_lval_idx(lval);
 	    assert(word_ix);
 	    assert(number_is_immediate(word_ix, IMM_WID, 0));
+	    assert(! number_is_unknown(word_ix));
 	    use_word = get_number_immediate(word_ix);
+	    /* This method no longer works since variable arrays do not
+	     * support <variable_id>_<idx> access any more. We need real
+	     * array specific opcodes (%assign/ar, etc.). */
+	    fprintf(stderr, "%s:%u: tgt-vvp sorry: non-blocking assignment "
+	            "to a real array word is not supported in V0.9.\n",
+	            ivl_expr_file(rval), ivl_expr_lineno(rval));
+	    exit(1);
       }
 
       if (del && (ivl_expr_type(del) == IVL_EX_DELAY)) {
@@ -742,7 +728,7 @@ static int show_stmt_assign_nb(ivl_statement_t net)
 		  fprintf(vvp_out, "    %%evctl%s %s, %d;\n", type, name,
 		                   count_index);
 		  clr_word(count_index);
-	    } else { 
+	    } else {
 		  fprintf(vvp_out, "    %%evctl/i %s, %lu;\n", name, count);
 	    }
       } else {
@@ -769,10 +755,39 @@ static int show_stmt_assign_nb(ivl_statement_t net)
       }
 
 
-      { struct vector_info res = draw_eval_expr(rval, 0);
-        unsigned wid = res.wid;
+      { struct vector_info res;
+	unsigned wid;
 	unsigned lidx;
 	unsigned cur_rbit = 0;
+	  /* Handle the special case that the expression is a real
+	     value. Evaluate the real expression, then convert the
+	     result to a vector. */
+	if (ivl_expr_value(rval) == IVL_VT_REAL) {
+	      int word = draw_eval_real(rval);
+	        /* This is the accumulated with of the l-value of the
+		   assignment. */
+	      wid = ivl_stmt_lwidth(net);
+
+	      res.base = allocate_vector(wid);
+	      res.wid = wid;
+
+	      if (res.base == 0) {
+		    fprintf(stderr, "%s:%u: vvp.tgt error: "
+			    "Unable to allocate %u thread bits for "
+			    "r-value expression.\n", ivl_expr_file(rval),
+			    ivl_expr_lineno(rval), wid);
+		    vvp_errors += 1;
+	      }
+
+	      fprintf(vvp_out, "    %%cvt/vr %u, %d, %u;\n",
+		      res.base, word, res.wid);
+
+	      clr_word(word);
+
+	} else {
+	      res = draw_eval_expr(rval, 0);
+	      wid = res.wid;
+	}
 
 	for (lidx = 0 ;  lidx < ivl_stmt_lvals(net) ;  lidx += 1) {
 	      unsigned bit_limit = wid - cur_rbit;
@@ -827,6 +842,8 @@ static int show_stmt_block_named(ivl_statement_t net, ivl_scope_t scope)
       fprintf(vvp_out, "    %%fork t_%u, S_%p;\n",
 	      sub_id, subscope);
       fprintf(vvp_out, "    %%jmp t_%u;\n", out_id);
+	/* Change the compiling scope to be the named blocks scope. */
+      fprintf(vvp_out, "    .scope S_%p;\n", subscope);
       fprintf(vvp_out, "t_%u ;\n", sub_id);
 
 	/* The statement within the fork is in a new thread, so no
@@ -835,6 +852,8 @@ static int show_stmt_block_named(ivl_statement_t net, ivl_scope_t scope)
 
       rc = show_stmt_block(net, subscope);
       fprintf(vvp_out, "    %%end;\n");
+	/* Return to the previous scope. */
+      fprintf(vvp_out, "    .scope S_%p;\n", scope);
 
       fprintf(vvp_out, "t_%u %%join;\n", out_id);
       clear_expression_lookaside();
@@ -846,8 +865,8 @@ static int show_stmt_block_named(ivl_statement_t net, ivl_scope_t scope)
 static int show_stmt_case(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
-      ivl_expr_t exp = ivl_stmt_cond_expr(net);
-      struct vector_info cond = draw_eval_expr(exp, 0);
+      ivl_expr_t expr = ivl_stmt_cond_expr(net);
+      struct vector_info cond = draw_eval_expr(expr, 0);
       unsigned count = ivl_stmt_case_count(net);
 
       unsigned local_base = local_count;
@@ -963,8 +982,8 @@ static int show_stmt_case(ivl_statement_t net, ivl_scope_t sscope)
 static int show_stmt_case_r(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
-      ivl_expr_t exp = ivl_stmt_cond_expr(net);
-      int cond = draw_eval_real(exp);
+      ivl_expr_t expr = ivl_stmt_cond_expr(net);
+      int cond = draw_eval_real(expr);
       unsigned count = ivl_stmt_case_count(net);
 
       unsigned local_base = local_count;
@@ -1105,11 +1124,13 @@ static void force_vector_to_lval(ivl_statement_t net, struct vector_info rvec)
 		  part_off = 0;
 	    } else {
 		  assert(number_is_immediate(part_off_ex, IMM_WID, 0));
+		  assert(! number_is_unknown(part_off_ex));
 		  part_off = get_number_immediate(part_off_ex);
 	    }
 
 	    if (word_idx != 0) {
 		  assert(number_is_immediate(word_idx, IMM_WID, 0));
+		  assert(! number_is_unknown(word_idx));
 		  use_word = get_number_immediate(word_idx);
 	    }
 
@@ -1124,7 +1145,6 @@ static void force_vector_to_lval(ivl_statement_t net, struct vector_info rvec)
 	    } else {
 		    /* Do not support bit or part selects of l-values yet. */
 		  assert(ivl_lval_mux(lval) == 0);
-		  assert(ivl_lval_part_off(lval) == 0);
 		  assert(ivl_lval_width(lval) == ivl_signal_width(lsig));
 
 		  assert((roff + use_wid) <= rvec.wid);
@@ -1173,9 +1193,15 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 	/* We do not currently support driving a signal to a bit or
 	 * part select (this could give us multiple drivers). */
       part_off_ex = ivl_lval_part_off(lval);
+	/* This should be verified in force_vector_to_lval() which is called
+	 * before this procedure. */
+      if (part_off_ex) {
+	    assert(number_is_immediate(part_off_ex, IMM_WID, 0));
+	    assert(! number_is_unknown(part_off_ex));
+      }
       if (ivl_signal_width(lsig) > ivl_signal_width(rsig) ||
           (part_off_ex && get_number_immediate(part_off_ex) != 0)) {
-	    fprintf(stderr, "%s:%u: vvp-tgt sorry: cannot %s signal to "
+	    fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s signal to "
 	            "a bit/part select.\n", ivl_expr_file(rval),
 	            ivl_expr_lineno(rval), command_name);
 	    exit(1);
@@ -1184,11 +1210,13 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 	/* At least for now, only handle force to fixed words of an array. */
       if ((lword_idx = ivl_lval_idx(lval)) != 0) {
 	    assert(number_is_immediate(lword_idx, IMM_WID, 0));
+	    assert(! number_is_unknown(lword_idx));
 	    use_lword = get_number_immediate(lword_idx);
       }
 
       if ((rword_idx = ivl_expr_oper1(rval)) != 0) {
 	    assert(number_is_immediate(rword_idx, IMM_WID, 0));
+	    assert(! number_is_unknown(rword_idx));
 	    use_rword = get_number_immediate(rword_idx);
       }
 
@@ -1272,11 +1300,13 @@ static int show_stmt_deassign(ivl_statement_t net)
 	    part_off = 0;
 	    if (part_off_ex != 0) {
 		  assert(number_is_immediate(part_off_ex, 64, 0));
+		  assert(! number_is_unknown(part_off_ex));
 		  part_off = get_number_immediate(part_off_ex);
 	    }
 
 	    if (word_idx != 0) {
 		  assert(number_is_immediate(word_idx, IMM_WID, 0));
+		  assert(! number_is_unknown(word_idx));
 		  use_word = get_number_immediate(word_idx);
 	    }
 
@@ -1292,9 +1322,9 @@ static int show_stmt_condit(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
       unsigned lab_false, lab_out;
-      ivl_expr_t exp = ivl_stmt_cond_expr(net);
+      ivl_expr_t expr = ivl_stmt_cond_expr(net);
       struct vector_info cond
-	    = draw_eval_expr(exp, STUFF_OK_XZ|STUFF_OK_47|STUFF_OK_RO);
+	    = draw_eval_expr(expr, STUFF_OK_XZ|STUFF_OK_47|STUFF_OK_RO);
 
       assert(cond.wid == 1);
 
@@ -1365,14 +1395,14 @@ static int show_stmt_delay(ivl_statement_t net, ivl_scope_t sscope)
 static int show_stmt_delayx(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
-      ivl_expr_t exp = ivl_stmt_delay_expr(net);
+      ivl_expr_t expr = ivl_stmt_delay_expr(net);
       ivl_statement_t stmt = ivl_stmt_sub_stmt(net);
 
-      switch (ivl_expr_value(exp)) {
+      switch (ivl_expr_value(expr)) {
 
 	  case IVL_VT_BOOL:
 	  case IVL_VT_LOGIC: {
-		struct vector_info del = draw_eval_expr(exp, 0);
+		struct vector_info del = draw_eval_expr(expr, 0);
 		fprintf(vvp_out, "    %%ix/get 0, %u, %u;\n",
 			del.base, del.wid);
 		clr_vector(del);
@@ -1380,7 +1410,7 @@ static int show_stmt_delayx(ivl_statement_t net, ivl_scope_t sscope)
 	  }
 
 	  case IVL_VT_REAL: {
-		int word = draw_eval_real(exp);
+		int word = draw_eval_real(expr);
 		fprintf(vvp_out, "    %%cvt/ir 0, %d;\n", word);
 		clr_word(word);
 		break;
@@ -1458,23 +1488,22 @@ static int show_stmt_fork(ivl_statement_t net, ivl_scope_t sscope)
       int rc = 0;
       unsigned cnt = ivl_stmt_block_count(net);
       ivl_scope_t scope = ivl_stmt_block_scope(net);
+      unsigned is_named = (scope != 0);
 
       unsigned out = transient_id++;
       unsigned id_base = transient_id;
 
 	/* cnt is the number of sub-threads. If the fork-join has no
 	   name, then we can put one of the sub-threads in the current
-	   thread, so decrement the count by one. */
-      if (scope == 0) {
+	   thread, so decrement the count by one and use the current
+	   scope for all the threads. */
+      if (! is_named) {
 	    cnt -= 1;
 	    scope = sscope;
       }
 
       transient_id += cnt;
 
-	/* If no subscope use provided */
-      if (!scope) scope = sscope;
-
 	/* Draw a fork statement for all but one of the threads of the
 	   fork/join. Send the threads off to a bit of code where they
 	   are implemented. */
@@ -1485,7 +1514,7 @@ static int show_stmt_fork(ivl_statement_t net, ivl_scope_t sscope)
 
 	/* If we are putting one sub-thread into the current thread,
 	   then draw its code here. */
-      if (ivl_stmt_block_scope(net) == 0)
+      if (! is_named)
 	    rc += show_statement(ivl_stmt_block_stmt(net, cnt), scope);
 
 
@@ -1495,6 +1524,8 @@ static int show_stmt_fork(ivl_statement_t net, ivl_scope_t sscope)
       }
       fprintf(vvp_out, "    %%jmp t_%u;\n", out);
 
+	/* Change the compiling scope to be the named forks scope. */
+      if (is_named) fprintf(vvp_out, "    .scope S_%p;\n", scope);
 	/* Generate the sub-threads themselves. */
       for (idx = 0 ;  idx < cnt ;  idx += 1) {
 	    fprintf(vvp_out, "t_%u ;\n", id_base+idx);
@@ -1502,6 +1533,8 @@ static int show_stmt_fork(ivl_statement_t net, ivl_scope_t sscope)
 	    rc += show_statement(ivl_stmt_block_stmt(net, idx), scope);
 	    fprintf(vvp_out, "    %%end;\n");
       }
+	/* Return to the previous scope. */
+      if (is_named) fprintf(vvp_out, "    .scope S_%p;\n", sscope);
 
 	/* This is the label for the out. Use this to branch around
 	   the implementations of all the child threads. */
@@ -1567,6 +1600,7 @@ static int show_stmt_release(ivl_statement_t net)
 	    part_off = 0;
 	    if (part_off_ex != 0) {
 		  assert(number_is_immediate(part_off_ex, 64, 0));
+		  assert(! number_is_unknown(part_off_ex));
 		  part_off = get_number_immediate(part_off_ex);
 	    }
 
@@ -1581,6 +1615,7 @@ static int show_stmt_release(ivl_statement_t net)
 
 	    if (word_idx != 0) {
 		  assert(number_is_immediate(word_idx, IMM_WID, 0));
+		  assert(! number_is_unknown(word_idx));
 		  use_word = get_number_immediate(word_idx);
 	    }
 
@@ -1597,9 +1632,9 @@ static int show_stmt_repeat(ivl_statement_t net, ivl_scope_t sscope)
 {
       int rc = 0;
       unsigned lab_top = local_count++, lab_out = local_count++;
-      ivl_expr_t exp = ivl_stmt_cond_expr(net);
-      struct vector_info cnt = draw_eval_expr(exp, 0);
-      char *sign = ivl_expr_signed(exp) ? "s" : "u";
+      ivl_expr_t expr = ivl_stmt_cond_expr(net);
+      struct vector_info cnt = draw_eval_expr(expr, 0);
+      char *sign = ivl_expr_signed(expr) ? "s" : "u";
 
 	/* Test that 0 < expr */
       fprintf(vvp_out, "T_%u.%u %%cmp/%s 0, %u, %u;\n", thread_count,
