@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -28,14 +28,11 @@
 # include  "vpi_priv.h"
 # include  "parse_misc.h"
 # include  "statistics.h"
-#ifdef HAVE_MALLOC_H
-# include  <malloc.h>
-#endif
 # include  <iostream>
 # include  <list>
-# include  <stdlib.h>
-# include  <string.h>
-# include  <assert.h>
+# include  <cstdlib>
+# include  <cstring>
+# include  <cassert>
 
 #ifdef __MINGW32__
 #include <windows.h>
@@ -67,7 +64,7 @@ enum operand_e {
 	/* The operand is a second functor pointer */
       OA_FUNC_PTR2,
 	/* The operand is a VPI handle */
-      OA_VPI_PTR,
+      OA_VPI_PTR
 };
 
 struct opcode_table_s {
@@ -141,7 +138,7 @@ const static struct opcode_table_s opcode_table[] = {
       { "%ix/get", of_IX_GET, 3,  {OA_BIT1,     OA_BIT2,     OA_NUMBER} },
       { "%ix/get/s",of_IX_GET_S,3,{OA_BIT1,     OA_BIT2,     OA_NUMBER} },
       { "%ix/getv",of_IX_GETV,2,  {OA_BIT1,     OA_FUNC_PTR, OA_NONE} },
-      { "%ix/getv/s",of_IX_GETVS,2, {OA_BIT1,   OA_FUNC_PTR, OA_NONE} },
+      { "%ix/getv/s",of_IX_GETV_S,2, {OA_BIT1,   OA_FUNC_PTR, OA_NONE} },
       { "%ix/load",of_IX_LOAD,3,  {OA_NUMBER,   OA_BIT1,     OA_BIT2} },
       { "%ix/mul", of_IX_MUL, 3,  {OA_NUMBER,   OA_BIT1,     OA_BIT2} },
       { "%ix/sub", of_IX_SUB, 3,  {OA_NUMBER,   OA_BIT1,     OA_BIT2} },
@@ -1370,6 +1367,8 @@ static struct __vpiModPathSrc*make_modpath_src(struct __vpiModPath*path,
 		  break;
 #endif
 		default:
+		  posedge = false;
+		  negedge = false;
 		  fprintf(stderr, "Unknown edge identifier %c(%d).\n", edge,
 		          edge);
 		  assert(0);
